# https://dev.entrouvert.org/issues/63188#note-1

import datetime
import os

import ipdb
import pandas as pd
from quixote import get_publisher
from utils import produce_csv
from wcs.formdef import FormDef
from wcs.qommon.storage import NotEqual

pub = get_publisher()
tenant = os.path.basename(pub.app_dir)
print('tenant', tenant)


def hours(time):
    return time.hour + time.minute / 60


# ventillation des demandes par formulaires, répartitions des fréquentations ...
def gen_formdef():
    for formdef in FormDef.select():
        count = formdef.data_class().count([NotEqual('status', 'draft')])
        age = pd.DataFrame(
            (
                (datetime.datetime.now() - datetime.datetime(*formdata.receipt_time[:6])).total_seconds()
                / 86400
            )
            for formdata in formdef.data_class().select([NotEqual('status', 'draft')], iterator=True)
        )
        receipt_time = pd.DataFrame(
            hours(datetime.datetime(*formdata.receipt_time[:6]).time())
            for formdata in formdef.data_class().select([NotEqual('status', 'draft')], iterator=True)
        )
        seen_files = set()
        disk_use = 0
        for formdata in formdef.data_class().select([NotEqual('status', 'draft')], iterator=True):
            for v in formdata.data.values():
                if hasattr(v, 'qfilename'):
                    if v.qfilename not in seen_files:
                        seen_files.add(v.qfilename)
                        disk_use += os.path.getsize(v.get_fs_filename())
        yield {
            'Catégorie': formdef.category.name if formdef.category else '',
            'Formulaire': formdef.name,
            'Workflow': formdef.workflow.name,
            'Nombre de statuts': len(formdef.workflow.possible_status),
            'Nombre de champs': len(formdef.fields or []),
            'Nombre de champs de traitement': len(formdef.workflow.get_backoffice_fields()),
            'Nombre de champs avec conditions': len(
                [field for field in formdef.fields or [] if field.condition]
            ),
            'Nombre de pages': len([field for field in formdef.fields or [] if field.key == 'page']),
            'Nombre de sauts sur expiration': len(
                [
                    item
                    for status in formdef.workflow.possible_status
                    for item in status.items or []
                    if getattr(item, 'timeout', None)
                ]
            ),
            'Nombre de sauts sur trigger': len(
                [
                    item
                    for status in formdef.workflow.possible_status
                    for item in status.items or []
                    if getattr(item, 'trigger', None)
                ]
            ),
            'Nombre de sauts sur condition': len(
                [
                    item
                    for status in formdef.workflow.possible_status
                    for item in status.items or []
                    if item.condition
                ]
            ),
            'Nombre de demandes': count,
            'Âge': age,
            'Heure de réception': receipt_time,
            'Usage du disque (en Mo)': disk_use / (1024 * 1024),
        }


with ipdb.launch_ipdb_on_exception():
    produce_csv(f'wcs-{tenant}-formdefs', gen_formdef)
